/**********************************************************************
*	main - Starting routine for 68332 Queued Serial Module demo program.
*
*	PARAMETERS PASSED:   NONE
*
*	PARAMETERS RETURNED: NONE
*
*	
***********************************************************************/

char MsgRx[32];

main()
{
    char *MsgTx = "Hello World                             ";

    QSM_setup();

    for(;;)
    {
           Load_TxBuf(MsgTx);
           Load_CmdBuf();
           SPCR1_Enable();
           UnLoad_RxBuf();
           SPCR1_Disable();
           Blank_TxBuf();
           Blank_CmdBuf();
           Blank_RxBuf();
    }
} 
